@@ -17,6 +17,8 @@ class TXAWSTestCase(TestCase):
             del os.environ["AWS_ACCESS_KEY_ID"]
         if "AWS_SECRET_ACCESS_KEY" in os.environ:
             del os.environ["AWS_SECRET_ACCESS_KEY"]
+        if "AWS_ENDPOINT" in os.environ:
+            del os.environ["AWS_ENDPOINT"]
 
     def _restore_environ(self):
         for key in set(os.environ) - set(self.orig_environ):
