@@ -246,20 +246,76 @@ sample_describe_security_groups_multiple_result = """\
           </ipRanges>
         </item>
         <item>
-          <ipProtocol>udp</ipProtocol>
-          <fromPort>81</fromPort>
-          <toPort>81</toPort>
+          <ipProtocol>tcp</ipProtocol>
+          <fromPort>0</fromPort>
+          <toPort>65535</toPort>
           <groups>
             <item>
               <userId>group-user-id</userId>
-              <groupName>group-name</groupName>
+              <groupName>group-name1</groupName>
             </item>
           </groups>
-          <ipRanges>
+          <ipRanges />
+        </item>
+        <item>
+          <ipProtocol>udp</ipProtocol>
+          <fromPort>0</fromPort>
+          <toPort>65535</toPort>
+          <groups>
             <item>
-              <cidrIp>0.0.0.0/16</cidrIp>
+              <userId>group-user-id</userId>
+              <groupName>group-name1</groupName>
             </item>
-          </ipRanges>
+          </groups>
+          <ipRanges />
+        </item>
+        <item>
+          <ipProtocol>icmp</ipProtocol>
+          <fromPort>-1</fromPort>
+          <toPort>-1</toPort>
+          <groups>
+            <item>
+              <userId>group-user-id</userId>
+              <groupName>group-name1</groupName>
+            </item>
+          </groups>
+          <ipRanges />
+        </item>
+        <item>
+          <ipProtocol>tcp</ipProtocol>
+          <fromPort>0</fromPort>
+          <toPort>65535</toPort>
+          <groups>
+            <item>
+              <userId>group-user-id</userId>
+              <groupName>group-name2</groupName>
+            </item>
+          </groups>
+          <ipRanges />
+        </item>
+        <item>
+          <ipProtocol>udp</ipProtocol>
+          <fromPort>0</fromPort>
+          <toPort>65535</toPort>
+          <groups>
+            <item>
+              <userId>group-user-id</userId>
+              <groupName>group-name2</groupName>
+            </item>
+          </groups>
+          <ipRanges />
+        </item>
+        <item>
+          <ipProtocol>icmp</ipProtocol>
+          <fromPort>-1</fromPort>
+          <toPort>-1</toPort>
+          <groups>
+            <item>
+              <userId>group-user-id</userId>
+              <groupName>group-name2</groupName>
+            </item>
+          </groups>
+          <ipRanges />
         </item>
       </ipPermissions>
     </item>
@@ -268,6 +324,93 @@ sample_describe_security_groups_multiple_result = """\
 """ % (version.ec2_api,)
 
 
+sample_describe_security_groups_multiple_groups = """\
+<?xml version="1.0"?>
+<DescribeSecurityGroupsResponse xmlns="http://ec2.amazonaws.com/doc/%s/">
+<requestId>481987ac-07e2-4f34-99b9-38bcce029ce9</requestId>
+<securityGroupInfo>
+  <item>
+    <ownerId>170743011661</ownerId>
+    <groupName>web/ssh</groupName>
+    <groupDescription>Web and SSH access</groupDescription>
+    <ipPermissions>
+      <item>
+        <ipProtocol>icmp</ipProtocol>
+        <fromPort>-1</fromPort>
+        <toPort>-1</toPort>
+        <groups>
+          <item>
+            <userId>170723411662</userId>
+            <groupName>default</groupName>
+          </item>
+          <item>
+            <userId>175723011368</userId>
+            <groupName>test1</groupName>
+          </item>
+        </groups>
+        <ipRanges/>
+      </item>
+      <item>
+        <ipProtocol>tcp</ipProtocol>
+        <fromPort>1</fromPort>
+        <toPort>65535</toPort>
+        <groups>
+          <item>
+            <userId>170723411662</userId>
+            <groupName>default</groupName>
+          </item>
+          <item>
+            <userId>175723011368</userId>
+            <groupName>test1</groupName>
+          </item>
+        </groups>
+        <ipRanges/>
+      </item>
+      <item>
+        <ipProtocol>udp</ipProtocol>
+        <fromPort>1</fromPort>
+        <toPort>65535</toPort>
+        <groups>
+          <item>
+            <userId>170723411662</userId>
+            <groupName>default</groupName>
+          </item>
+          <item>
+            <userId>175723011368</userId>
+            <groupName>test1</groupName>
+          </item>
+        </groups>
+        <ipRanges/>
+      </item>
+      <item>
+        <ipProtocol>tcp</ipProtocol>
+        <fromPort>22</fromPort>
+        <toPort>22</toPort>
+        <groups/>
+        <ipRanges>
+          <item>
+            <cidrIp>0.0.0.0/0</cidrIp>
+          </item>
+        </ipRanges>
+      </item>
+      <item>
+        <ipProtocol>tcp</ipProtocol>
+        <fromPort>80</fromPort>
+        <toPort>80</toPort>
+        <groups/>
+        <ipRanges>
+          <item>
+            <cidrIp>0.0.0.0/0</cidrIp>
+          </item>
+        </ipRanges>
+      </item>
+    </ipPermissions>
+  </item>
+</securityGroupInfo>
+</DescribeSecurityGroupsResponse>
+""" % (version.ec2_api,)
+
+
 sample_create_security_group = """\
 <CreateSecurityGroupResponse xmlns="http://ec2.amazonaws.com/doc/%s/">
   <return>true</return>
@@ -563,6 +706,15 @@ sample_duplicate_keypair_result = """\
 """
 
 
+sample_import_keypair_result = """\
+<?xml version="1.0"?>
+<ImportKeyPairResponse xmlns="http://ec2.amazonaws.com/doc/%s/">
+  <keyName>example-key-name</keyName>
+  <keyFingerprint>1f:51:ae:28:bf:89:e9:d8:1f:25:5d:37:2d:7d:b8:ca:9f:f5:f1:6f</keyFingerprint>
+</ImportKeyPairResponse>
+""" % (version.ec2_api,)
+
+
 sample_allocate_address_result = """\
 <?xml version="1.0"?>
 <AllocateAddressResponse xmlns="http://ec2.amazonaws.com/doc/%s/">
@@ -735,6 +887,18 @@ sample_get_bucket_result = """\
 </ListBucketResult>
 """ % (version.s3_api,)
 
+
+sample_get_bucket_location_result = """\
+<LocationConstraint xmlns="http://s3.amazonaws.com/doc/2006-03-01/">EU</LocationConstraint>
+
+"""
+sample_request_payment = """\
+<?xml version="1.0" encoding="UTF-8"?>
+<RequestPaymentConfiguration xmlns="http://s3.amazonaws.com/doc/2006-03-01/">
+  <Payer>Requester</Payer>
+</RequestPaymentConfiguration>
+"""
+
 sample_s3_signature_mismatch = """\
 <?xml version="1.0" encoding="UTF-8"?>
 <Error>
@@ -760,3 +924,28 @@ sample_s3_invalid_access_key_result = """\
   <AWSAccessKeyId>SOMEKEYID</AWSAccessKeyId>
 </Error>
 """
+
+sample_access_control_policy_result = """\
+<AccessControlPolicy>
+  <Owner>
+    <ID>8a6925ce4adf588a4f21c32aa37900beef</ID>
+    <DisplayName>baz@example.net</DisplayName>
+  </Owner>
+  <AccessControlList>
+    <Grant>
+      <Grantee xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:type="CanonicalUser">
+        <ID>8a6925ce4adf588a4f21c32aa379004fef</ID>
+        <DisplayName>foo@example.net</DisplayName>
+      </Grantee>
+      <Permission>FULL_CONTROL</Permission>
+    </Grant>
+    <Grant>
+      <Grantee xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:type="CanonicalUser">
+        <ID>8a6925ce4adf588a4f21c32aa37900feed</ID>
+        <DisplayName>bar@example.net</DisplayName>
+      </Grantee>
+      <Permission>READ</Permission>
+    </Grant>
+  </AccessControlList>
+</AccessControlPolicy>"""
+
